@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -37,7 +37,9 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
+#ifdef HAVE_POLL_H
 #include <poll.h>
+#endif
 
 #include "gmime-gpg-context.h"
 #include "gmime-filter-charset.h"
@@ -309,7 +311,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	const char *charset;
 	GMimeStream *stream;
 	
-	gpg = g_new (struct _GpgCtx, 1);
+	gpg = g_slice_new (struct _GpgCtx);
 	gpg->mode = GPG_CTX_MODE_SIGN;
 	gpg->session = session;
 	g_object_ref (session);
@@ -473,7 +475,7 @@ static void
 gpg_ctx_free (struct _GpgCtx *gpg)
 {
 	GMimeSigner *signer, *next;
-	int i;
+	guint i;
 	
 	if (gpg->session)
 		g_object_unref (gpg->session);
@@ -528,7 +530,7 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 		signer = next;
 	}
 	
-	g_free (gpg);
+	g_slice_free (struct _GpgCtx, gpg);
 }
 
 static const char *
@@ -564,7 +566,7 @@ gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, char **sfd, int passwd_fd,
 	const char *hash_str;
 	GPtrArray *argv;
 	char *buf;
-	int i;
+	guint i;
 	
 	argv = g_ptr_array_new ();
 	g_ptr_array_add (argv, "gpg");
@@ -791,7 +793,7 @@ next_token (char *in, char **token)
 		inptr++;
 	
 	if (token)
-		*token = g_strndup (start, inptr - start);
+		*token = g_strndup (start, (size_t) (inptr - start));
 	
 	return inptr;
 }
@@ -1180,27 +1182,49 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 	return 0;
 }
 
-static size_t
+#ifdef ALLOC_NEAREST_POW2
+static inline size_t
 nearest_pow (size_t num)
 {
-	size_t n = num > 0 ? num - 1 : 0;
+	size_t n;
 	
+	if (num == 0)
+		return 0;
+	
+	n = num - 1;
+#if defined (__GNUC__) && defined (__i386__)
+	__asm__("bsrl %1,%0\n\t"
+		"jnz 1f\n\t"
+		"movl $-1,%0\n"
+		"1:" : "=r" (n) : "rm" (n));
+	n = (1 << (n + 1));
+#else
 	n |= n >> 1;
 	n |= n >> 2;
 	n |= n >> 4;
 	n |= n >> 8;
 	n |= n >> 16;
 	n++;
+#endif
 	
 	return n;
 }
 
+#define next_alloc_size(n) nearest_pow (n)
+#else
+static inline size_t
+next_alloc_size (size_t n)
+{
+	return ((n + 63) / 64) * 64;
+}
+#endif
+
 #define status_backup(gpg, start, len) G_STMT_START {                     \
 	if (gpg->statusleft <= len) {                                     \
 		size_t slen, soff;                                        \
 		                                                          \
 		soff = gpg->statusptr - gpg->statusbuf;                   \
-		slen = nearest_pow (soff + len + 1);                      \
+		slen = next_alloc_size (soff + len + 1);                  \
 		                                                          \
 		gpg->statusbuf = g_realloc (gpg->statusbuf, slen);        \
 		gpg->statusptr = gpg->statusbuf + soff;                   \
@@ -1221,6 +1245,83 @@ enum {
 	GPG_N_FDS
 };
 
+#ifndef HAVE_POLL
+struct pollfd {
+	int fd;
+	short events;
+	short revents;
+};
+
+#define POLLIN   (1 << 0)
+#define POLLPRI  (1 << 1)
+#define POLLOUT  (1 << 2)
+#define POLLERR  (1 << 3)
+#define POLLHUP  (1 << 4)
+#define POLLNVAL (1 << 5)
+
+#ifdef HAVE_SELECT
+static int
+poll (struct pollfd *pfds, nfds_t nfds, int timeout)
+{
+	fd_set rset, wset, xset;
+	struct timeval tv;
+	int maxfd = 0;
+	int ready;
+	nfds_t i;
+	
+	if (nfds == 0)
+		return 0;
+	
+	/* initialize our select() timeout */
+	tv.tv_sec = timeout / 1000;
+	tv.tv_usec = (timeout % 1000) * 1000;
+	
+	/* initialize our select() fd sets */
+	FD_ZERO (&rset);
+	FD_ZERO (&wset);
+	FD_ZERO (&xset);
+	
+	for (i = 0; i < nfds; i++) {
+		if (pfds[i].events & POLLIN)
+			FD_SET (pfds[i].fd, &rset);
+		if (pfds[i].events & POLLOUT)
+			FD_SET (pfds[i].fd, &wset);
+		if (pfds[i].events & POLLPRI)
+			FD_SET (pfds[i].fd, &xset);
+		if (pfds[i].fd > maxfd && (pfds[i].events & (POLLIN | POLLOUT | POLLPRI)))
+			maxfd = pfds[i].fd;
+		pfds[i].revents = 0;
+	}
+	
+	/* poll our fds... */
+	if ((ready = select (maxfd + 1, &rset, &wset, &xset, timeout != -1 ? &tv : NULL)) > 0) {
+		ready = 0;
+		
+		for (i = 0; i < nfds; i++) {
+			if (FD_ISSET (pfds[i].fd, &rset))
+				pfds[i].revents |= POLLIN;
+			if (FD_ISSET (pfds[i].fd, &wset))
+				pfds[i].revents |= POLLOUT;
+			if (FD_ISSET (pfds[i].fd, &xset))
+				pfds[i].revents |= POLLPRI;
+			
+			if (pfds[i].revents != 0)
+				ready++;
+		}
+	}
+	
+	return ready;
+}
+#else
+static int
+poll (struct pollfd *pfds, nfds_t nfds, int timeout)
+{
+	errno = EIO;
+	return -1;
+}
+#endif /* HAVE_SELECT */
+#endif /* ! HAVE_POLL */
+
 static int
 gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 {
@@ -1290,7 +1391,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 			goto exception;
 		
 		if (nread > 0) {
-			status_backup (gpg, buffer, nread);
+			status_backup (gpg, buffer, (size_t) nread);
 			if (gpg_ctx_parse_status (gpg, err) == -1)
 				return -1;
 		} else {
@@ -1338,8 +1439,8 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 	}
 	
 	if ((pfds[GPG_PASSWD_FD].revents & (POLLOUT | POLLHUP)) && gpg->need_passwd && gpg->send_passwd) {
-		ssize_t w, nwritten = 0;
-		size_t n;
+		size_t n, nwritten = 0;
+		ssize_t w;
 		
 		d(printf ("sending gpg our passphrase...\n"));
 		
@@ -1707,7 +1808,7 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 {
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
-	int i;
+	guint i;
 	
 	gpg = gpg_ctx_new (context->session, ctx->path);
 	if (sign)
@@ -1884,7 +1985,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 {
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
-	int i;
+	guint i;
 	
 	gpg = gpg_ctx_new (context->session, ctx->path);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_EXPORT);
