@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -192,7 +192,6 @@ static void
 g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 {
 	GMimeHeaderList *headers = ((GMimeObject *) message)->headers;
-	InternetAddressList *list;
 	guint i;
 	
 	message->from = NULL;
@@ -572,7 +571,7 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 			g_string_append (str, "\n\t");
 			len = 1;
 		} else if (lwsp) {
-			g_string_append_len (str, lwsp, part->start - lwsp);
+			g_string_append_len (str, lwsp, (size_t) (part->start - lwsp));
 		}
 		
 		g_string_append_len (str, part->start, part->len);
@@ -707,9 +706,9 @@ process_header (GMimeObject *object, int action, const char *header, const char
 {
 	GMimeMessage *message = (GMimeMessage *) object;	
 	InternetAddressList *addrlist;
-	int offset, i;
 	time_t date;
-	gulong id;
+	int offset;
+	guint i;
 	
 	for (i = 0; i < G_N_ELEMENTS (message_headers); i++) {
 		if (!g_ascii_strcasecmp (message_headers[i], header))
@@ -862,7 +861,7 @@ message_remove_header (GMimeObject *object, const char *header)
 	GMimeMessage *message = (GMimeMessage *) object;
 	InternetAddressList *addrlist;
 	GMimeRecipientType type;
-	int i;
+	guint i;
 	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return FALSE;
@@ -1021,7 +1020,7 @@ g_mime_message_new (gboolean pretty_headers)
 {
 	GMimeHeaderList *headers;
 	GMimeMessage *message;
-	int i;
+	guint i;
 	
 	message = g_object_newv (GMIME_TYPE_MESSAGE, 0, NULL);
 	
