@@ -26,16 +26,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <regex.h>
 
 #include "gmime-parser.h"
 
 #include "gmime-table-private.h"
-#include "gmime-stream-mem.h"
 #include "gmime-message-part.h"
+#include "gmime-parse-utils.h"
+#include "gmime-stream-mem.h"
 #include "gmime-multipart.h"
 #include "gmime-part.h"
 
+#if GLIB_MAJOR_VERSION > 2 || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 14)
+#define HAVE_GLIB_REGEX
+#elif defined (HAVE_REGEX_H)
+#include <regex.h>
+#endif
+
 #ifdef ENABLE_WARNINGS
 #define w(x) x
 #else
@@ -127,10 +133,15 @@ struct _GMimeParserPrivate {
 	gint64 from_offset;
 	GByteArray *from_line;
 	
-	regex_t header_regex;
 	GMimeParserHeaderRegexFunc header_cb;
 	gpointer user_data;
 	
+#if defined (HAVE_GLIB_REGEX)
+	GRegex *regex;
+#elif defined (HAVE_REGEX_H)
+	regex_t regex;
+#endif
+	
 	/* header buffer */
 	char *headerbuf;
 	char *headerptr;
@@ -289,6 +300,10 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	parser->priv = g_new (struct _GMimeParserPrivate, 1);
 	parser_init (parser, NULL);
 	
+#if defined (HAVE_GLIB_REGEX)
+	parser->priv->regex = NULL;
+#endif
+	
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
 	parser->priv->persist_stream = TRUE;
@@ -302,8 +317,13 @@ g_mime_parser_finalize (GObject *object)
 	
 	parser_close (parser);
 	
+#if defined (HAVE_GLIB_REGEX)
+	if (parser->priv->regex)
+		g_regex_unref (parser->priv->regex);
+#elif defined (HAVE_REGEX_H)
 	if (parser->priv->have_regex)
-		regfree (&parser->priv->header_regex);
+		regfree (&parser->priv->regex);
+#endif
 	
 	g_free (parser->priv);
 	
@@ -571,6 +591,9 @@ g_mime_parser_set_respect_content_length (GMimeParser *parser, gboolean respect_
  * Sets the regular expression pattern @regex on @parser. Whenever a
  * header matching the pattern @regex is parsed, @header_cb is called
  * with @user_data as the user_data argument.
+ *
+ * If @regex is %NULL, then the previously registered regex callback
+ * is unregistered and no new callback is set.
  **/
 void
 g_mime_parser_set_header_regex (GMimeParser *parser, const char *regex,
@@ -582,10 +605,17 @@ g_mime_parser_set_header_regex (GMimeParser *parser, const char *regex,
 	
 	priv = parser->priv;
 	
+#if defined (HAVE_GLIB_REGEX)
+	if (priv->regex) {
+		g_regex_unref (priv->regex);
+		priv->regex = NULL;
+	}
+#elif defined (HAVE_REGEX_H)
 	if (priv->have_regex) {
-		regfree (&priv->header_regex);
+		regfree (&priv->regex);
 		priv->have_regex = FALSE;
 	}
+#endif
 	
 	if (!regex || !header_cb)
 		return;
@@ -593,7 +623,11 @@ g_mime_parser_set_header_regex (GMimeParser *parser, const char *regex,
 	priv->header_cb = header_cb;
 	priv->user_data = user_data;
 	
-	priv->have_regex = !regcomp (&priv->header_regex, regex, REG_EXTENDED | REG_ICASE | REG_NOSUB);
+#if defined (HAVE_GLIB_REGEX)
+	priv->regex = g_regex_new (regex, G_REGEX_RAW | G_REGEX_EXTENDED | G_REGEX_CASELESS, 0, NULL);
+#elif defined (HAVE_REGEX_H)
+	priv->have_regex = !regcomp (&priv->regex, regex, REG_EXTENDED | REG_ICASE | REG_NOSUB);
+#endif
 }
 
 
@@ -642,7 +676,7 @@ parser_fill (GMimeParser *parser, size_t atleast)
 		priv->inend += nread;
 	}
 	
-	return priv->inend - priv->inptr;
+	return (ssize_t) (priv->inend - priv->inptr);
 }
 
 
@@ -732,12 +766,12 @@ parser_step_from (GMimeParser *parser)
 			
 			if (inptr + 1 >= inend) {
 				/* we don't have enough data; if we can't get more we have to bail */
-				left = inend - start;
+				left = (ssize_t) (inend - start);
 				priv->inptr = start;
 				goto refill;
 			}
 			
-			len = inptr - start;
+			len = (size_t) (inptr - start);
 			inptr++;
 			
 			if (len >= 5 && !strncmp (start, "From ", 5)) {
@@ -883,7 +917,7 @@ header_parse (GMimeParser *parser, HeaderRaw **tail)
 			end = inptr;
 	}
 	
-	header->value = g_strndup (start, end - start);
+	header->value = g_strndup (start, (size_t) (end - start));
 	
 	header->offset = priv->header_offset;
 	
@@ -893,10 +927,16 @@ header_parse (GMimeParser *parser, HeaderRaw **tail)
 	priv->headerleft += priv->headerptr - priv->headerbuf;
 	priv->headerptr = priv->headerbuf;
 	
+#if defined (HAVE_GLIB_REGEX)
+	if (priv->regex && g_regex_match (priv->regex, header->name, 0, NULL))
+		priv->header_cb (parser, header->name, header->value,
+				 header->offset, priv->user_data);
+#elif defined (HAVE_REGEX_H)
 	if (priv->have_regex &&
 	    !regexec (&priv->header_regex, header->name, 0, NULL, 0))
 		priv->header_cb (parser, header->name, header->value,
 				 header->offset, priv->user_data);
+#endif
 }
 
 static int
@@ -965,7 +1005,7 @@ parser_step_headers (GMimeParser *parser)
 					
 					if (inptr == inend) {
 						/* don't have the full field name */
-						left = inend - start;
+						left = (ssize_t) (inend - start);
 						priv->inptr = start;
 						goto refill;
 					}
@@ -1009,8 +1049,8 @@ parser_step_headers (GMimeParser *parser)
 				
 				raw_header_append (priv, start, len);
 				header_append (priv, start, len);
+				left = (ssize_t) (inend - inptr);
 				priv->midline = TRUE;
-				left = inend - inptr;
 				priv->inptr = inptr;
 				goto refill;
 			}
@@ -1033,7 +1073,7 @@ parser_step_headers (GMimeParser *parser)
 			inptr++;
 		}
 		
-		left = inend - inptr;
+		left = (ssize_t) (inend - inptr);
 		priv->inptr = inptr;
 	} while (1);
 	
@@ -1110,7 +1150,6 @@ static ContentType *
 parser_content_type (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register const char *inptr;
 	ContentType *content_type;
 	const char *value;
 	
@@ -1277,7 +1316,7 @@ check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
 #define MAX_BOUNDARY_LEN(bounds) (bounds ? bounds->boundarylenmax + 2 : 0)
 
 static int
-parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
+parser_scan_content (GMimeParser *parser, GByteArray *content, guint *crlf)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	register char *inptr;
@@ -1379,7 +1418,7 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream;
 	gint64 start, end;
-	int crlf;
+	guint crlf;
 	
 	g_assert (priv->state >= GMIME_PARSER_STATE_HEADERS_END);
 	
@@ -1529,8 +1568,9 @@ static int
 parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboolean preface)
 {
 	GByteArray *buffer;
-	int found, crlf;
 	char *face;
+	guint crlf;
+	int found;
 	
 	buffer = g_byte_array_new ();
 	found = parser_scan_content (parser, buffer, &crlf);
